import nest
import numpy as np
import sys
import os
import matplotlib.pyplot as plt
sys.path.insert(0, 'code/NEST_model/')
from params19Oct2023 import *

#------------------------------------------------
# FILE LABELS
label= '/home/nest/' #'/home/nest/data/NEST_model/'
memTestLabel='4memPotTest_'+pltLabel+'.png'
fmemTestOut=os.path.join(label, memTestLabel)
fmemPotIz1='4iz_membrane_potential_1_'+str(pltLabel)
fmemPotIz2='4iz_membrane_potential_2_'+str(pltLabel)
fmemPotIz3='4iz_membrane_potential_3_'+str(pltLabel)
fmemPotIz4='4iz_membrane_potential_4_'+str(pltLabel)
fspikesIZ1='4iz_spikes_1_'+str(pltLabel)
fspikesIZ2='4iz_spikes_2_'+str(pltLabel)
fspikesIZ3='4iz_spikes_3_'+str(pltLabel)
fspikesIZ4='4iz_spikes_4_'+str(pltLabel)


#-------------------------------------------
# KERNEL
nest.ResetKernel()
seed = [np.random.randint(0, 9999999)] * num_threads #nest.SetKernelStatus({'rng_seed': 12345})
nest.SetKernelStatus({'resolution': sim_resolution,  
                      'print_time': True,
                      'rng_seeds': seed,
                      'local_num_threads': num_threads, 
                      'overwrite_files': True})

#-------------------------------------------
# SPIKE GENERATORS: EXTERNAL STIM

s_ex1 = nest.Create("spike_generator", params={"spike_times": spTimes_exc1}) # Exc Generator #1
s_ex2 = nest.Create("spike_generator", params={"spike_times": spTimes_exc2}) # Exc Generator #2
s_in = nest.Create("spike_generator", params={"spike_times": spTimes_inh}) # Inh Generator #2


#-------------------------------------------
# NEURON POPs
n_iz2 = nest.Create("izhikevich", 4)
nest.SetStatus(n_iz2, params=exc_neuron_params)


#-------------------------------------------
# MULTIMETER
mm_iz2 = nest.Create("multimeter", n=4, params={
    "interval": sim_resolution,
    'record_from': ['V_m'],
    'withgid': True, 
    'withtime': True, 
    'to_memory': True, #False,
    'precision': 17,  
    'to_file': True, #False}) #
    'label': os.path.join(label, fmemPotIz2)}) #"my_multimeter"}) 
    

# SPIKEDETECTOR
spkDet_iz2 = nest.Create("spike_detector", n=4, params={
    'withgid': True, 
    'withtime': True,
    'to_memory': True, #False, 
    'to_file': True, #False})
    'label': os.path.join(label, fspikesIZ2)}) #'spikes')})#nest.GetStatus(spikeDetectors,['events'])



#-------------------------------------------
# CONNECTIONS

# Connect 3 Spike Gen as Stimulus (2 w/ Exc Syn & 1 w/Inh Syn) 
nest.Connect(s_ex1, [n_iz2[0]], syn_spec={"weight": W_exc, 'delay': 1.0})
nest.Connect(s_ex2, [n_iz2[0]], syn_spec={"weight": W_exc, 'delay': 1.0})
nest.Connect(s_in, [n_iz2[0]], syn_spec={"weight": W_inh, 'delay': 1.0})

# Connect 4 Neurons 
nest.Connect([n_iz2[0]], [n_iz2[1]], syn_spec={'weight': 20.0, 'delay': 55.0}) # Neuron-A to -C
nest.Connect([n_iz2[0]], [n_iz2[2]], syn_spec={'weight': 20.0, 'delay': 65.0}) # Neuron-A to -C
nest.Connect([n_iz2[1]], [n_iz2[3]], syn_spec={'weight': 10.0, 'delay': 66.0}) # Neuron-B to -D
nest.Connect([n_iz2[2]], [n_iz2[3]], syn_spec={'weight': 10.0, 'delay': 55.0}) # Neuron-C to -D

# Connect Multimeter, Spike Detector for Vm, Spikes
nest.Connect(mm_iz2, n_iz2,'one_to_one')
nest.Connect(n_iz2, spkDet_iz2,'one_to_one')


#-------------------------------------------
# SIMULATE
nest.Simulate(sim_time) # 120 x 1000 ms


#--------------------
# PREP PLOTS

#External Stim for Display
spExWeights1=np.repeat(W_exc,len(spTimes_exc1)) #nest.GetStatus(s_ex,'spike_weights')
spExWeights2=np.repeat(W_exc,len(spTimes_exc2)) #nest.GetStatus(s_ex,'spike_weights')
spInWeights=np.repeat(W_inh,len(spTimes_inh)) #nest.GetStatus(s_in,'spike_weights')



# Extract Vm and Time (all neurons interlaced)
eventsMemIzA = nest.GetStatus(mm_iz2)[0]["events"]
vIzA=eventsMemIzA['V_m']
tIzA=eventsMemIzA['times']
eventsMemIzB = nest.GetStatus(mm_iz2)[1]["events"]
vIzB=eventsMemIzB['V_m']
tIzB=eventsMemIzB['times']
eventsMemIzC = nest.GetStatus(mm_iz2)[2]["events"]
vIzC=eventsMemIzC['V_m']
tIzC=eventsMemIzC['times']
eventsMemIzD = nest.GetStatus(mm_iz2)[3]["events"]
vIzD=eventsMemIzD['V_m']
tIzD=eventsMemIzD['times']

# Extract Spike Senders, Times (all neurons interlaced)
izSpEvent = nest.GetStatus(spkDet_iz2,keys="events")
sendersIdxA=izSpEvent[0]["senders"]
timesA=izSpEvent[0]["times"]
sendersIdxB=izSpEvent[1]["senders"]
timesB=izSpEvent[1]["times"]
sendersIdxC=izSpEvent[2]["senders"]
timesC=izSpEvent[2]["times"]
sendersIdxD=izSpEvent[3]["senders"]
timesD=izSpEvent[3]["times"]

print(vIzA)
print(tIzA)


#--------------------
# PLOTS


fig1 = plt.figure(figsize=(10,8)) #fig1, axs = plt.subplots(5, 1, sharex=True) 


# Display Ext Stim, Times
axs0=plt.subplot(511) 
axs0.plot(spTimes_exc1, spExWeights1, 'g^', spTimes_exc2, spExWeights2, 'c^', spTimes_inh, spInWeights, 'rv')
axs0.set_xlim([0, 1000]) 
axs0.get_xaxis().set_ticks([])
axs0.set_ylim([(W_inh-8), (W_exc+8)])
axs0.get_yaxis().set_ticks([])
axs0.set_xlabel("time (ms)")
axs0.set_ylabel("Inputs (I)")
axs0.legend(("EX1","EX2", "IN"),loc='upper right')


# NEURON-A Vm & Spikes
axs1=plt.subplot(512) 
axs1.plot(tIzA, vIzA,'b-')
axs2 = axs1.twinx()
axs2.plot(timesA,np.repeat(1.0,len(timesA)),'b*',label='A Spikes')
axs1.set_xlim([0, 1000]) 
axs1.get_xaxis().set_ticks([])
axs1.set_ylim([-80.0, +40.0])
axs2.set_ylim([0.0, 1.05])
axs1.get_yaxis().set_ticks([])
axs2.get_yaxis().set_ticks([])
axs1.set_ylabel("V_m")
axs1.set_xlabel("time (ms)")
axs2.legend(loc='upper right')


# NEURON-B Vm & Spikes
axs3=plt.subplot(513) 
axs4 = axs3.twinx()
axs3.plot(tIzB, vIzB,'g-')
axs4.plot(timesB,np.repeat(1.0,len(timesB)),'g*',label='B Spikes')
axs3.set_xlim([0, 1000]) 
axs3.get_xaxis().set_ticks([])
#axs3.set_ylim([-80.0, +0.0])
axs4.set_ylim([0.0, 1.05])
axs3.get_yaxis().set_ticks([])
axs4.get_yaxis().set_ticks([])
axs3.set_ylabel("V_m")
axs3.set_xlabel("time (ms)")
axs4.legend(loc='upper right')


# NEURON-C Vm & Spikes
axs5=plt.subplot(514) 
axs6 = axs5.twinx()
axs5.plot(tIzC, vIzC,'c-')
axs6.plot(timesC,np.repeat(1.0,len(timesC)),'c*',label='C Spikes')
axs5.set_xlim([0, 1000]) 
axs5.get_xaxis().set_ticks([])
#axs5.set_ylim([-80.0, +0.0])
axs6.set_ylim([0.0, 1.05])
axs5.get_yaxis().set_ticks([])
axs6.get_yaxis().set_ticks([])
axs5.set_ylabel("V_m")
#axs[4].set_xlabel("time (ms)")
axs6.legend(loc='upper right') 


# NEURON-D Vm & Spikes
axs7=plt.subplot(515) 
axs8 = axs7.twinx()
axs7.plot(tIzD, vIzD,'m-')
axs8.plot(timesD,np.repeat(1.0,len(timesD)),'m*',label='D Spikes')
axs7.set_xlim([0, 1000]) 
#axs7.set_ylim([-80.0, +0.0])
axs8.set_ylim([0.0, 1.05])
axs7.get_yaxis().set_ticks([])
axs8.get_yaxis().set_ticks([])
axs7.set_ylabel("V_m")
axs7.set_xlabel("time (ms)")
axs8.legend(loc='upper right') 

#fig1.tight_layout() # (pad=2.0)
fig1.subplots_adjust(hspace=0)

plt.savefig("memNeur1Test12.png")
plt.show()
#plt.savefig("xxxx.png")
